@@ -9,6 +9,9 @@ from .UnivariateHindcastMargin import *
 
 from _c_ext_bivarmargins import ffi, lib as C_CALL
 
+from deprecated import deprecated
+from warnings import warn
+
 class BivariateHindcastMargin(object):
   """Main class for risk calculations in a time-collapsed 2-area hindcast model.
   It receives demand and wind generation data as opposed to just net demand data, to make it posible to calculate
@@ -28,7 +31,7 @@ class BivariateHindcastMargin(object):
   def __init__(self,demand,renewables,gen_dists,**kwargs):
 
     self.net_demand = np.ascontiguousarray((demand - renewables),dtype=np.int64).clip(min=0) #no negative net demand
-    self.wind = renewables
+    self.renewables = renewables
     self.demand = np.ascontiguousarray(demand,dtype=np.int64)
     self.gen_dists = self._parse_gendists(gen_dists,**kwargs)
     self.n = self.net_demand.shape[0]
@@ -83,6 +86,8 @@ class BivariateHindcastMargin(object):
                       np.int64(origin[0]),
                       np.int64(origin[1]),
                       np.int64(length),
+                      np.int64(bigen.X1.min),
+                      np.int64(bigen.X2.min),
                       np.int64(bigen.X1.max),
                       np.int64(bigen.X2.max),
                       ffi.cast("double *",bigen.X1.cdf_vals.ctypes.data),
@@ -263,7 +268,8 @@ class BivariateHindcastMargin(object):
 
     self.demand = np.flip(self.demand,axis=1)
     self.net_demand = np.flip(self.net_demand,axis=1)
-    self.gen_dists = self.gen_dists[::-1]
+    self.gen_dists = [self.gen_dists[1],self.gen_dists[0]]
+    #self.gen_dists = self.gen_dists[::-1]
 
   def lole(self,c,policy,axis=0,get_pointwise_risk=False):
     """calculate LOLE for one of the areas
@@ -276,24 +282,29 @@ class BivariateHindcastMargin(object):
 
     `axis (`int*): area index for which this will be calculated
 
+    `get_pointwise_risk (`bool`): If `True`, returns pandas dataframe with LOLE contributions historic observation
+
     """
 
-    if axis == 1:
-      self._swap_axes()
+    # if axis == 1:
+    #   self._swap_axes()
 
-    m = (-1,np.Inf)
+    m = [np.Inf,np.Inf]
+    m[axis] = -1
+    #m = (-1,np.Inf)
     lolp_vals = self.cdf(m=m,c=c,policy=policy,get_pointwise_risk=get_pointwise_risk) 
     if get_pointwise_risk:
-      return lolp_vals
+      lole = lolp_vals
     else:
-      return self.n * lolp_vals
+      lole = self.n * lolp_vals
+
+    return lole
 
     #lole = self._lole(c=c,policy=policy,get_pointwise_risk=get_pointwise_risk)
 
-    if axis == 1:
-      self._swap_axes()
+    # if axis == 1:
+    #   self._swap_axes()
 
-    return lole
 
   # @deprecated(version="1.0.0",reason="use lole() instead")
   # def lole(self,c,policy,axis=0,get_pointwise_risk=False):
@@ -348,6 +359,8 @@ class BivariateHindcastMargin(object):
 
     `axis` (`int`): area index for which this will be calculated 
 
+    `get_pointwise_risk (`bool`): If `True`, returns pandas dataframe with EEU contributions for each historic observation
+
     """
 
     if axis == 1:
@@ -393,11 +406,11 @@ class BivariateHindcastMargin(object):
 
     `policy` (`str`): Either 'share' or 'veto'
 
-    `get_pointwise_risk` (`str`): return pandas DataFrame with net demand values and corresponding pointwise risk measurements
+    `get_pointwise_risk` (`str`): return pandas DataFrame with EPU for each historic observation
 
     """
     if c > 0 or get_pointwise_risk:
-      self._check_null_fc()
+      #self._check_null_fc()
       X1 = self.gen_dists[0]
       X2 = self.gen_dists[1]
       n = self.n
@@ -465,7 +478,100 @@ class BivariateHindcastMargin(object):
 
     return self.net_demand[np.random.choice(range(self.n),size=n),:]
 
-  def efc(self,c,policy,metric="LOLE",axis=0,**kwargs):
+  def efc(self,**kwargs):
+    """This method calculates the equivalent firm capacity of interconnection for a given power system. It has been deprecated; use the `itc_efc` method instead. When called, this function calls `itc_efc`, passing all arguments along.
+    """
+    warn("This method is deprecated; use methods itc_efc or convgen_ifc instead.")
+    return self.itc_efc(**kwargs)
+
+  def convgen_efc(self, cap, prob, gen_axis, fc_axis, c,policy,metric="lole",axis=0,tol=0.1,**kwargs):
+    """Returns the amount of firm capacity that needs to be added to area `fc_axis` such that area `axis` has the same risk (as defined by `metric`) than if new conventional generation was installed in `gen_axis`.
+
+    **Parameters**:
+    
+    `cap` (`int`): maximum available capacity of new generator in MW
+
+    `prob` (`float`): availability probability for the new generator
+
+    `gen_axis` (`int`): Axis to which the new generator will be added
+
+    `fc_axis` (`int`): Area to which firm capacity will be added
+
+    `c` (`int`): interconnection capacity
+
+    `policy` (`str`): Either 'share' or 'veto'
+
+    `axis` (`int`): area for which risk will be calculated
+
+    `metric` (`string`): name of the instance's method that will be used to measure risk. Only 'LOLE' and 'EEU' are supported
+
+    `tol` (`float`): absolute error tolerance from true target risk value
+
+    """
+    if not (axis in [0,1]):
+      raise Exception("axis value must be 0 or 1")
+
+    if not (gen_axis in [0,1]):
+      raise Exception("gen_axis value must be 0 or 1")
+
+    if not (fc_axis in [0,1]):
+      raise Exception("fc_axis value must be 0 or 1")
+
+    if prob > 1 or prob < 0:
+      raise Exception("Availability value must be between 0 and 1")
+
+    other = 1 - gen_axis
+    gen_data = self.gen_dists[gen_axis].original_data
+
+    # create augmented conv gen distribution
+    new_row = pd.Series([cap,prob],index=["Capacity","Availability"])
+    augmented_data = gen_data.append(new_row,ignore_index=True)
+    augmented_gen_dist = ConvGenDistribution(augmented_data)
+
+    # create augmented conv gen bivariate distribution and get risk metric in agumented system
+    new_bivariate_dist = [0,1] #place holder values
+    new_bivariate_dist[gen_axis] = augmented_gen_dist
+    new_bivariate_dist[other] = self.gen_dists[other]
+    new_margin_dist = BivariateHindcastMargin(self.demand,self.renewables,new_bivariate_dist)
+    new_metric_func = getattr(new_margin_dist,metric)
+    new_metric_val = new_metric_func(c=c,policy=policy,axis=axis)
+
+    print("new metric val: {x}".format(x=new_metric_val))
+    ### take original system and add firm capacity until we get new_metric_val
+    # define bisection algorithm's bounds
+    if cap >= 0:
+      leftmost = 0
+      rightmost = cap
+    else:
+      rightmost = 0
+      leftmost = cap
+
+    # clone object to prevent any side effects
+    original_bivariate_dist = [ConvGenDistribution(self.gen_dists[area].original_data) for area in range(2)]
+    
+    # define objective for bisection algorithm
+    def find_efc(x):
+      # add firm capacity
+      original_bivariate_dist[fc_axis] += x
+      # create bivariate margin distribution object
+      dist = BivariateHindcastMargin(self.demand,self.renewables,original_bivariate_dist)
+      metric_func = getattr(dist,metric)
+      metric_val = metric_func(c=c,policy=policy,axis=axis)
+
+      # reset firm capacity to 0
+      original_bivariate_dist[fc_axis] += (-original_bivariate_dist[fc_axis].fc)
+
+      print("Adding {x}, getting val {v}".format(x=x,v=metric_val))
+      return metric_val - new_metric_val
+
+    efc, res = bisect(f=find_efc,a=leftmost,b=rightmost,full_output=True,xtol=tol/2,rtol=tol/(2*new_metric_val))
+    if not res.converged:
+      print("Warning: EFC estimator did not converge.")
+    #print("efc:{efc}".format(efc=efc))
+    return int(efc)
+
+
+  def itc_efc(self,c,policy,metric="lole",axis=0,tol=0.1,**kwargs):
     """Returns equivalent firm capacity of interconnector in one area
 
     **Parameters**:
@@ -474,23 +580,27 @@ class BivariateHindcastMargin(object):
 
     `policy` (`str`): Either 'share' or 'veto'
 
-    `axis` (`int`): area for which this will be calculated
+    `axis` (`int`): area for which risk will be calculated
 
     `metric` (`string`): name of the instance's method that will be used to measure risk
 
+    `tol` (`float`): absolute error tolerance from true target risk value
+
     """
-    if not metric in ["LOLE","EEU"]:
-      raise Exception("Only LOLE or EPU supported as risk metrics")
+    if not metric in ["lole","eeu"]:
+      raise Exception("Only 'lole' or 'eeu' supported as risk metrics")
     
+    #target value
     with_itc = getattr(self,metric)(c=c,axis=axis,policy=policy,**kwargs)
 
     def compare_itc_to_fc(k):
       self.gen_dists[axis] += k ## add firm capacity
       univar = UnivariateHindcastMargin(self.gen_dists[axis],self.net_demand[:,axis])
       #risk_metric = getattr(univar,metric)
-      k_fc_risk =  with_itc - getattr(univar,metric)()
+      without_itc = getattr(univar,metric)()
+      #k_fc_risk =  with_itc - without_itc
       self.gen_dists[axis] += (-k) #reset firm capacity to 0
-      return k_fc_risk
+      return with_itc - without_itc
 
     diff_to_null = compare_itc_to_fc(0)
 
@@ -513,11 +623,11 @@ class BivariateHindcastMargin(object):
           rightmost += c
       
       #print("finding efc in [{a},{b}]".format(a=leftmost,b=rightmost))
-      efc, res = bisect(f=compare_itc_to_fc,a=leftmost,b=rightmost,full_output=True)
+      efc, res = bisect(f=compare_itc_to_fc,a=leftmost,b=rightmost,full_output=True,xtol=tol/2,rtol=tol/(2*with_itc))
       if not res.converged:
         print("Warning: EFC estimator did not converge.")
       #print("efc:{efc}".format(efc=efc))
-      return efc
+      return int(efc)
 
   def _trapezoid_prob(self,X,ulc,c):
 
@@ -538,6 +648,8 @@ class BivariateHindcastMargin(object):
                         np.int64(ulc1),
                         np.int64(ulc2),
                         np.int64(c),
+                        np.int64(X.X1.min),
+                        np.int64(X.X2.min),
                         np.int64(X.X1.max),
                         np.int64(X.X2.max),
                         ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
@@ -570,6 +682,8 @@ class BivariateHindcastMargin(object):
                     np.int64(v1),
                     np.int64(v2),
                     np.int64(c),
+                    np.int64(FX1.min),
+                    np.int64(FX2.min),
                     np.int64(FX1.max),
                     np.int64(FX2.max),
                     ffi.cast("double *",FX1.cdf_vals.ctypes.data),
@@ -597,6 +711,8 @@ class BivariateHindcastMargin(object):
                     np.int64(v1),
                     np.int64(v2),
                     np.int64(c),
+                    np.int64(FX1.min),
+                    np.int64(FX2.min),
                     np.int64(FX1.max),
                     np.int64(FX2.max),
                     ffi.cast("double *",FX1.cdf_vals.ctypes.data),
@@ -681,6 +797,8 @@ class BivariateHindcastMargin(object):
       C_CALL.region_simulation(
         np.int64(n),
         ffi.cast("long *",simulated.ctypes.data),
+        np.int64(X.X1.min),
+        np.int64(X.X2.min),
         np.int64(X.X1.max),
         np.int64(X.X2.max),
         ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
@@ -762,6 +880,8 @@ class BivariateHindcastMargin(object):
       C_CALL.conditioned_simulation(
           np.int64(n),
           ffi.cast("long *",simulated.ctypes.data),
+          np.int64(X.X1.min),
+          np.int64(X.X2.min),
           np.int64(X.X1.max),
           np.int64(X.X2.max),
           ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
@@ -792,18 +912,18 @@ class BivariateHindcastMargin(object):
 
     `policy` (`str`): Either 'share' or 'veto'
 
-    `get_pointwise_risk` (`str`): return pandas DataFrame with net demand values and corresponding point-wise risk measurements.
+    `get_pointwise_risk` (`str`): return pandas DataFrame with shortfall probabilities induced by each historic observation
 
     """
 
     m = np.clip(m,a_min=-self.MARGIN_BOUND,a_max=self.MARGIN_BOUND)
     m1, m2 = m
-    self._check_null_fc()
+    #self._check_null_fc()
 
-    X1 = self.gen_dists[0]
-    X2 = self.gen_dists[1]
+    gendist1, gendist2 = self.gen_dists
+    #X2 = self.gen_dists[1]
 
-    X = BivariateConvGenDist(X1,X2) #system-wide conv. gen. distribution
+    #X = BivariateConvGenDist(X1,X2) #system-wide conv. gen. distribution
    
     n = self.n
 
@@ -821,10 +941,12 @@ class BivariateHindcastMargin(object):
       d1, d2 = self.demand[i,:]
 
       point_cdf = C_CALL.get_cond_cdf(
-                      np.int64(X.X1.max),
-                      np.int64(X.X2.max),
-                      ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
-                      ffi.cast("double *",X.X2.cdf_vals.ctypes.data),
+                      np.int64(gendist1.min),
+                      np.int64(gendist2.min),
+                      np.int64(gendist1.max),
+                      np.int64(gendist2.max),
+                      ffi.cast("double *",gendist1.cdf_vals.ctypes.data),
+                      ffi.cast("double *",gendist2.cdf_vals.ctypes.data),
                       np.int64(m1),
                       np.int64(m2),
                       np.int64(v1),
@@ -834,6 +956,7 @@ class BivariateHindcastMargin(object):
                       np.int64(c),
                       np.int64(policy == "share"))
 
+      #print("point cdf: {x}, index: {i}".format(x=point_cdf, i=i))
       cdf += point_cdf
 
       #print(v1)
@@ -864,7 +987,7 @@ class BivariateHindcastMargin(object):
 
     `policy` (`str`): Either 'share' or 'veto'
 
-    `get_pointwise_risk` (`str`): return pandas DataFrame with net demand values and corresponding pointwise risk measurements
+    `get_pointwise_risk` (`str`): return pandas DataFrame with LOLPs for each historic observation
 
     """
 
@@ -946,7 +1069,7 @@ class BivariateHindcastMargin(object):
 
     `policy` (`str`): Either 'share' or 'veto'
 
-    `get_pointwise_risk` (`str`): return pandas DataFrame with net demand values and corresponding pointwise risk measurements
+    `get_pointwise_risk` (`str`): return pandas DataFrame with system LOLE contributions for each historic observation
     """
     return self.n * self.system_lolp(c,get_pointwise_risk)
 
